import { ModelDefinition } from "./model_definitions";
import { Collection, ObjectId, Filter, Sort } from "mongodb";
import SafeMongo from ".";
export interface ModelDataBase {
    _id: ObjectId;
    _v: number;
}
type FilterProperties<T, U> = {
    [key in keyof T]: T[key] extends U ? key : never;
};
type AllowedNames<T, U> = FilterProperties<T, U>[keyof T];
export declare class InvalidObjectError extends Error {
    constructor();
}
export default class Model<T extends ModelDataBase> {
    private _sm;
    private _definition;
    private _collection;
    private _version;
    private _new;
    private _fetched;
    get version(): number;
    get collection(): Promise<Collection<any>>;
    constructor(_sm: SafeMongo, _definition: ModelDefinition);
    new(data?: Partial<T>): T;
    validate(data: T): void;
    save(data: T): Promise<T>;
    findById(id: ObjectId | string): Promise<T | null>;
    findOne(filter: Filter<T>): Promise<T | null>;
    find(filter: Filter<T>, limit?: number, skip?: number, sort?: Sort): Promise<T[]>;
    delete(doc: T | ObjectId | string): Promise<void>;
    deleteFilter(filter: Filter<T>): Promise<void>;
    incement(elm: ObjectId | string | T, field: AllowedNames<T, number>, amount?: number): Promise<void>;
    private _add_fetched;
    private _upgrade;
    private _validate;
}
export declare function recursiveDeepCopy<T = any | any[]>(o: T): T;
export {};
